#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)         *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import math

from __logging__ import logger, abort
from E3TextInputFile import E3TextInputFile
from E3RootTreeWriter import E3RootTreeWriter


class E3CernEvent(dict):

    """ Event data structure for the original CERN-style EEE data files.

    This is essentially a python dictionary with the keys specified below.
    """

    FIELDS = [('RunNumber'   , 'I'),
              ('EventNumber' , 'I'),
              ('Seconds'     , 'I'),
              ('Nanoseconds' , 'I'),
              ('Timestamp'   , 'D'),
              ('DeltaTime'   , 'D'),
              ('XDir'        , 'D'),
              ('YDir'        , 'D'),
              ('ZDir'        , 'D'),
              ('Theta'       , 'D'),
              ('Phi'         , 'D'),
              ('ChiSquare'   , 'D'),
              ('TimeOfFlight', 'D'),
              ('TrackLength' , 'D')
              ]

    def __init__(self, data):
        """ Constructor.
        """
        self['RunNumber'] = int(data[0])
        self['EventNumber'] = int(data[1])
        self['Seconds'] = int(data[2])
        self['Nanoseconds'] = int(data[3])
        self['Timestamp'] = self['Seconds'] + 1e-9*self['Nanoseconds']
        self['DeltaTime'] = 0.
        self['XDir'] = float(data[5])
        self['YDir'] = float(data[6])
        self['ZDir'] = float(data[7])
        self['Theta'] = math.degrees(math.acos(self['ZDir']))
        self['Phi'] = math.degrees(math.atan2(self['YDir'], self['XDir']))
        self['ChiSquare'] = float(data[8])
        self['TimeOfFlight'] = float(data[9])
        self['TrackLength'] = float(data[10])

    def __str__(self):
        """ String formatting.
        """
        return 'Event %d--%d (t = %.9f)' %\
            (self['RunNumber'], self['EventNumber'], self['Timestamp'])



class E3CernNtuple(E3TextInputFile, E3RootTreeWriter):
    
    """ Basic data file descriptor for the original CERN-style EEE data files.
    """

    FILE_EXTENSION = '.out'

    def __init__(self, filePath):
        """ Constructor.
        """
        E3TextInputFile.__init__(self, filePath)
        self.PreviousEvent = None

    def next(self):
        """ And this is the main iteration function.
        """
        data = self.FileObject.next().split()
        event = E3CernEvent(data)
        if self.PreviousEvent is not None:
            event['DeltaTime'] =\
                float(event['Timestamp'] - self.PreviousEvent['Timestamp'])
        self.PreviousEvent = event
        return event

    def writeRootTuple(self, outputFilePath = None):
        """ Write the content of the file into an output tuple.
        """
        if outputFilePath is None:
            outputFilePath = self.FilePath.replace(self.FILE_EXTENSION, '.root')
        E3RootTreeWriter.__init__(self, outputFilePath, 'CernTuple')
        for (branchName, branchType) in E3CernEvent.FIELDS:
            self.addBranch(branchName, branchType)
        branches = [item[0] for item in E3CernEvent.FIELDS]
        logger.info('Writing ntuple rows to the ROOT tree...')
        for event in self:
            for branchName in branches:
                self.setValue(branchName, event[branchName])
            self.fill()
        E3RootTreeWriter.close(self)


if __name__ == '__main__':
    from __eee__ import DATA_FOLDER
    filePath = os.path.join(DATA_FOLDER, 'sample_cerntuple.out')
    inputFile = E3CernNtuple(filePath)
    print(inputFile.isCernNtuple())
    print(inputFile.isTextNtuple())
    for i in range(10):
        print(inputFile.next())
    inputFile.close()
