#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)         *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import os
import math

from __logging__ import logger, abort
from E3TextInputFile import E3TextInputFile
from E3RootTreeWriter import E3RootTreeWriter


class E3TextNtuple(E3TextInputFile, E3RootTreeWriter):
    
    """ Basic data file descriptor for a generic E3TextNtuple.
    """

    FILE_EXTENSION = '.out'

    def __init__(self, filePath):
        """ Constructor.
        """
        E3TextInputFile.__init__(self, filePath)
        logger.info('Parsing file header...')
        if not self.Header.startswith('E3TextNtuple:'):
            abort('Malformed file header, this isn\'t a text ntuple.')
        self.TreeName = self.Header.split(':')[1]
        logger.info('Done, tree name is "%s".' % self.TreeName)
        self.Fields = self.FileObject.next().strip('\r\n').split()
        self.Branches = []
        self.Formatters = []
        for field in self.Fields:
            branchName, branchType = field.split('/')
            self.Branches.append(branchName)
            if branchType in ['F', 'D']:
                self.Formatters.append(float)
            else:
                self.Formatters.append(int)

    def next(self):
        """ And this is the main iteration function.
        """
        data = self.FileObject.next().split()
        event = {}
        for (i, branchName) in enumerate(self.Branches):
            event[branchName] = self.Formatters[i](data[i])
        return event

    def writeRootTuple(self, outputFilePath = None):
        """ Write the content of the file into an output tuple.
        """
        if outputFilePath is None:
            outputFilePath = self.FilePath.replace(self.FILE_EXTENSION, '.root')
        E3RootTreeWriter.__init__(self, outputFilePath, self.TreeName)
        for branchTitle in self.Fields:
            (branchName, branchType) = branchTitle.split('/')
            self.addBranch(branchName, branchType)
        logger.info('Writing ntuple rows to the ROOT tree...')
        for event in self:
            for branchName in self.Branches:
                self.setValue(branchName, event[branchName])
            self.fill()
        E3RootTreeWriter.close(self)



if __name__ == '__main__':
    from __eee__ import DATA_FOLDER
    filePath = os.path.join(DATA_FOLDER, 'sample_coincidences_free.out')
    inputFile = E3TextNtuple(filePath)
    print(inputFile.isTextNtuple())
    print(inputFile.isCernNtuple())
    for i in range(10):
        print(inputFile.next())
    inputFile.close()
