#!/usr/bin/env python
# *********************************************************************
# * Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)         *
# *                                                                   *
# * For the license terms see the file LICENCE, distributed           *
# * along with this software.                                         *
# *********************************************************************
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.


import ROOT
ROOT.gROOT.SetStyle('Plain')

from __logging__ import logger
from __geometry__ import DETECTOR_GEOMETRY



class TRPCStrip(ROOT.TGeoVolume):

    """ A single RPC strip.
    """

    WIDTH = 0.8*DETECTOR_GEOMETRY.STRIP_WIDTH
    LENGTH = DETECTOR_GEOMETRY.STRIP_LENGTH
    HEIGHT = DETECTOR_GEOMETRY.STRIP_THICKNESS
    COLOR  = ROOT.kGray + 2

    def __init__(self):
        """
        """
        self.Shape = ROOT.TGeoBBox('RPC Strip', 0.5*self.WIDTH,
                                   0.5*self.LENGTH, 0.5*self.HEIGHT)
        ROOT.TGeoVolume.__init__(self, 'RPC Strip volume', self.Shape)
        self.SetLineColor(self.COLOR)



class TRPCPlane(ROOT.TGeoVolume):
    
    """ A detector plane.
    """

    NUM_STRIPS = DETECTOR_GEOMETRY.NUM_STRIPS_PER_PLANE
    WIDTH = DETECTOR_GEOMETRY.STRIP_WIDTH*NUM_STRIPS
    LENGTH = TRPCStrip.LENGTH
    HEIGHT = TRPCStrip.HEIGHT

    def __init__(self):
        """
        """
        self.Shape = ROOT.TGeoBBox('RPC Plane', 0.5*self.WIDTH,
                                   0.5*self.LENGTH, 0.5*self.HEIGHT)
        ROOT.TGeoVolume.__init__(self, 'RPC Plane volume', self.Shape)
        self.StripVol = TRPCStrip()
        for i in xrange(self.NUM_STRIPS):
            dx = DETECTOR_GEOMETRY.STRIP_WIDTH*(i - self.NUM_STRIPS/2)
            shift = ROOT.TGeoTranslation(dx, 0., 0.)
            self.AddNode(self.StripVol, i, shift)



class TEEEDetector(ROOT.TGeoVolume):

    """ The full three-plane detector.
    """

    WIDTH = TRPCPlane.WIDTH
    LENGTH = TRPCPlane.LENGTH
    HEIGHT = DETECTOR_GEOMETRY.TOP_Z

    def __init__(self):
        """
        """
        self.Shape = ROOT.TGeoBBox('EEE Detector', 0.5*self.WIDTH,
                                   0.5*self.LENGTH, 0.5*self.HEIGHT)
        ROOT.TGeoVolume.__init__(self, 'EEE Detector volume', self.Shape)
        self.PlaneVol = TRPCPlane()
        shift = ROOT.TGeoTranslation(0., 0., DETECTOR_GEOMETRY.BOT_Z)
        self.AddNode(self.PlaneVol, 0, shift)
        shift = ROOT.TGeoTranslation(0., 0., DETECTOR_GEOMETRY.MID_Z)
        self.AddNode(self.PlaneVol, 1, shift)
        shift = ROOT.TGeoTranslation(0., 0., DETECTOR_GEOMETRY.TOP_Z)
        self.AddNode(self.PlaneVol, 2, shift)



if __name__ == '__main__':
    geo = ROOT.TGeoManager('Geometry manager', 'Geometry manager')
    eee = TEEEDetector()
    geo.SetTopVolume(eee)
    geo.CloseGeometry()
    geo.SetVisLevel(4)
    geo.SetVisOption(0)
    eee.Draw()
    raw_input('Press enter to exit...')
