/*
  Copyright (C) 2012--2013 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef E3DigiEvent_H
#define E3DigiEvent_H

#include <vector>
#include <iomanip>

#include "E3.h"
#include "E3MessageLogger.h"
#include "E3BaseEvent.h"
#include "E3StatusCode.h"
#include "E3DigiWord.h"
#include "E3NinoMap.h"
#include "E3StripData.h"
#include "E3RunInfo.h"


/*! \class E3DigiEvent
  \brief Base class for the overall event structure.

  This class encapsulates all the low-level event digis (i.e., it is
  essentially a collection of raw numbers from the DAQ).
*/

class E3DigiEvent : public E3BaseEvent
{
 public:
  /// \brief Default constructor.
  E3DigiEvent();

  /// \brief Alternative constructor passing along the seconds field.
  E3DigiEvent(E3::UnixTime_t unixTime);

  /// \brief Destructor.
  ~E3DigiEvent() {}

  /// \brief Clear up everything.
  void clear();

  /// \brief Unpack the underlying binary data.
  StatusCode unpack(const E3RunInfo& runInfo);

  /// \brief Return the unix time.
  inline E3::UnixTime_t unixTime() const
  { return m_unixTime; }

  ///\brief Return the time (in seconds) since the start of the run.
  inline double timeSinceStartRun() const 
  { return m_timeSinceStartRun; }

  /// \brief Return the second counter.
  inline unsigned long secondCount() const
  { return m_secondCount; }
  
  /// \brief Return the nanosecond counter.
  inline unsigned long nanosecondCount() const
  { return m_nanosecondCount; }

  /// \brief Return the PPS count.
  inline unsigned long ppsCount() const
  { return m_ppsCount; }

  /// \brief Return the m_triggerId class member.
  inline E3::Trigger_t triggerId() const
  { return m_triggerId; }

  /// \brief Access the m_digiWordVec class member.
  inline E3DigiWordVec digiWordVec() const
  { return m_digiWordVec; }

  /// \brief Add a data word to the m_digiWordVec vector.
  void addDigiWord(E3::DigiWord_t digi);

  /// \brief Add a data word to the m_digiWordVec vector.
  void addDigiWord(E3DigiWord digi);

  /// \brief Add some strip data to the m_stripDataVec class member.
  void addStripData(E3::Plane_t plane, E3StripData data);

  /// \brief Return the number of strip data for a given plane.
  unsigned short numStripData(E3::Plane_t plane) const;

  /// \brief Return the total number of strip data in the event.
  unsigned short numStripData() const;

  /// \brief Retrieve the overall event size in bytes.
  unsigned int size() const;
  
  /// \bried Standard output.
  std::ostream& fillStream(std::ostream& os) const;

  /// \brief Standard output.
  friend std::ostream& operator<<(std::ostream& os, const E3DigiEvent& e)
  { return e.fillStream(os); }

 protected:
  /// \brief The seconds since January 1, 1970 from the host pc running the DAQ.
  E3::UnixTime_t m_unixTime;

  /// \brief The time since the start of the run.
  double m_timeSinceStartRun;

  /// \brief The value of the second counter. 
  unsigned long m_secondCount;

  /// \brief The value of the nanosecond counter.
  unsigned long m_nanosecondCount;

  /// \brief The value of the nanosecond counter latched at the arrival time
  /// of the last PPS from the GPS.
  unsigned long m_ppsCount;
  
  /// \brief The trigger number.
  E3::Trigger_t m_triggerId;

  /// \brief The basic vector of digi words.
  E3DigiWordVec m_digiWordVec;

  /// \brief The vector of strip data.
  E3StripDataVec m_stripDataVec[3];

  /// \brief Log an unpacking error.
  void unpackingError(E3::Board_t board, const E3DigiWordVecIter &digi,
                      const std::ostringstream &msg);
};


/// \brief Temporary ad-hoc container to map the digi words within an event
/// into a representation that can be easily used to build up the E3StripData
/// structures.
/// 
/// Here we are effectively creating a structure that in python would look
/// something like:
/// digiMap[(plane, channel)] = [side, digiWord]
typedef std::pair<E3::Plane_t, E3::Channel_t> E3DigiMapKey;

typedef std::vector< std::pair<bool, E3DigiWord> > E3DigiMapValue;

typedef E3DigiMapValue::const_iterator E3DigiMapValueIter;

typedef std::map<E3DigiMapKey, E3DigiMapValue> E3DigiMap;

typedef E3DigiMap::const_iterator E3DigiMapIter;

#endif
