/*
  Copyright (C) 2012--2013 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef E3DigiWord_H
#define E3DigiWord_H

#include <iostream>
#include <vector>
#include <map>

#include "E3.h"


/// \brief Markers for several different types of digi information.
enum DigiWordType {
  BOARD_BEGIN  = 0xbbbbbbbb,
  BOARD_END    = 0xbebebebe,
  TDC_BEGIN    = 0x00ebebeb,
  TDC_END      = 0x00eeeeee,
  RISING_EDGE  = 0x40000000,
  FALLING_EDGE = 0x50000000
};


/*! \class E3DigiWord
  \brief Base class describing a digi word.

  A digi is a binary data word encapsulating a specific piece of information
  (e.g. the arrival time of a given rising edge for a specific channel
  relative to the start of the event readout).
  
  Note that we provide all the interfaces for the different types of digi
  word in the same class, rather than defining different classes. It is
  responsibility of the user to make sure that the particular bit manipulation
  performed in a given interface makes sense for that particular type of digi.
  This might ask for some re-thinking in the future.
*/

class E3DigiWord
{
 public:
  /// \brief Full constructor from a binary data word.
  E3DigiWord(E3::DigiWord_t payload);

  /// \brief Destructor.
  ~E3DigiWord() {}

  /// \brief Return true if the digi is a board begin.
  inline bool isBoardBegin() const
  { return m_payload == BOARD_BEGIN; }

  /// \brief Return true if the digi is a board end.
  inline bool isBoardEnd() const 
  { return m_payload == BOARD_END; }

  /// \brief Return true if the digi is a TDC begin.
  inline bool isTdcBegin() const
  { return (m_payload & 0x00ffffff) == TDC_BEGIN; }

  /// \brief Return true if the digi is a TDC end.
  inline bool isTdcEnd() const
  { return (m_payload & 0x00ffffff) == TDC_END; }

  /// \brief Return true if the digi is a TDC end for a given TDC.
  inline bool isTdcEnd(E3::Tdc_t tdc) const
  { return m_payload == TDC_END + (tdc << 24); }

  /// \brief Return true if the digi is a rising edge.
  inline bool isRisingEdge() const
  { return (m_payload & 0xf0000000) == RISING_EDGE; }

  /// \brief Return true if the digi is a falling edge.
  inline bool isFallingEdge() const
  { return (m_payload & 0xf0000000) == FALLING_EDGE; }

  /// \brief Return true if the digi corresponds to an edge.
  inline bool isEdge() const
  { return isRisingEdge() || isFallingEdge(); }

  /// \brief Return the plain payload.
  inline E3::DigiWord_t payload() const
  { return m_payload; }

  /// \brief Return the channel value for an edge digi.
  E3::Channel_t channel() const;

  /// \brief Return the time of an edge since the start of the event readout.
  /// 
  /// This is measured in units of 96 ps.
  inline unsigned long ps() const
  { return 96*(m_payload & 0x7ffff); }

  /// \brief Return the unpacked nanosecond field.
  ///
  /// The 16 MSBs are incremented in units of 4000 clock ticks (100 us at the
  /// nominal 40 MHz clock frequency), while the 16 LSBs are incremented in
  /// units of clock ticks (25 ns at the nominal 40 MHz frequency).
  inline unsigned long ns(double clockPeriod = 25.) const
  { return (unsigned long)(4000*clockPeriod*((m_payload >> 16) & 0xffff) +
                           clockPeriod*(m_payload & 0xffff)); }

  /// \brief
  inline unsigned long ppsns(double clockPeriod = 25.) const
  { return (unsigned long)(clockPeriod*m_payload); }

  /// \brief Return the TDC ID for a TDC begin field.
  inline E3::Tdc_t tdc() const
  { return m_payload >> 24; }

  /// \brief Standard output.
  std::ostream& fillStream(std::ostream& os) const;

  /// \brief Standard output.
  friend std::ostream& operator<<(std::ostream& os, const E3DigiWord& w)
    { return w.fillStream(os); }

 private:
  /// \brief The actual digi word.
  E3::DigiWord_t m_payload;
};


typedef std::vector<E3DigiWord> E3DigiWordVec;
typedef std::vector<E3DigiWord>::const_iterator E3DigiWordVecIter;

#endif
