/*
  Copyright (C) 2012--2013 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/


#include "E3Event.h"


E3Event::E3Event()
{
  clear();
}


E3Event::E3Event(E3::UnixTime_t unixTime)
{
  clear();
  m_unixTime = unixTime;
}


void E3Event::clear()
{
  E3DigiEvent::clear();
  for (E3::Plane_t plane = 0; plane < E3::kNumPlanes; plane++) {
    m_hitVec[plane].clear();
    m_clusterVec[plane].clear();
  }
  m_trackVec.clear();
}


unsigned short E3Event::numHits(E3::Plane_t plane) const
{
  return m_hitVec[plane].size();
}


unsigned short E3Event::numHits() const
{
  unsigned short n = 0;
  for (E3::Plane_t plane = 0; plane < 3; plane++) {
    n += numHits(plane);
  }
  return n;
}


unsigned short E3Event::numClusters(E3::Plane_t plane) const
{
  return m_clusterVec[plane].size();
}


unsigned short E3Event::numClusters() const
{
  unsigned short n = 0;
  for (E3::Plane_t plane = 0; plane < 3; plane++) {
    n += numClusters(plane);
  }
  return n;
}


unsigned short E3Event::numTracks() const
{
  return m_trackVec.size();
}


E3Track E3Event::bestTrack() const
{
  if (numTracks()) return m_trackVec.front();
  else return E3Track();
}


StatusCode E3Event::reconstruct(const E3TelescopeConfiguration& configuration)
{
  StatusCode sc;
  if ((sc = findHits(configuration))) return sc;
  if ((sc = findClusters())) return sc;
  if ((sc = fitTracks())) return sc;
  return StatusCode::SUCCESS;
}


StatusCode E3Event::findHits(const E3TelescopeConfiguration& config)
{
  // Initialize some variables.
  E3StripDataVecIter stripData;
  E3DigiWordVecIter posEdge, negEdge;
  E3DigiWordVec posVec, negVec;
  unsigned long posPs, negPs;
  int dPs;
  double x, y, z, t;
  double tot[2];
  // Loop over the telescope planes...
  for (E3::Plane_t plane = 0; plane < E3::kNumPlanes; plane++) {
    // Loop over the strip data in the plane... 
    for (stripData = m_stripDataVec[plane].begin();
         stripData != m_stripDataVec[plane].end(); stripData++) {
      // Double loop over the edges on the two plane end for the strip data.
      posVec = stripData->posEdgeDigiVec();
      negVec = stripData->negEdgeDigiVec();
      for (posEdge = posVec.begin(); posEdge != posVec.end(); posEdge++){
        for (negEdge = negVec.begin(); negEdge != negVec.end(); negEdge++){
          if (posEdge->isRisingEdge() && negEdge->isRisingEdge()) {
            // At this point we're testing all the possible pairs of rising
            // edges on the opposite end of each strip with data.
            // 
            // Correct the timing of the rising edges so that the difference
            // in the cable length is compensated for.
            posPs = posEdge->ps() + config.cableDelay(plane, E3::kPosEnd);
            negPs = negEdge->ps() + config.cableDelay(plane, E3::kNegEnd);
            // Calculate the time difference of the rising edges and
            // convert it into the coordinate along the strip.
            dPs = posPs - negPs;
            y = dPs*config.propSpeed();
            // Is the x coordinate within the physical range?
            if (fabs(y) < config.ymax()) {
              x = (stripData->channel() - config.yoff())*config.stripPitch();
              z = config.z(stripData->plane());
              t = 0.001*(posPs - config.stripLength()/config.propSpeed());
              // Calculate the TOT (this is done between pairs in the same
              // plane ends so that the cable length is irrelevant).
              if ((posEdge + 1)->isFallingEdge()) {
                tot[E3::kPosEnd] = 0.001*((posEdge + 1)->ps() - posEdge->ps());
              } else {
                tot[E3::kPosEnd] = -1;
              }
              if ((negEdge + 1)->isFallingEdge()) {
                tot[E3::kNegEnd] = 0.001*((negEdge + 1)->ps() - negEdge->ps());
              } else {
                tot[E3::kNegEnd] = -1;
              }
              E3Hit hit(stripData->plane(), stripData->channel(), x, y, z, t,
                        tot);
              m_hitVec[plane].push_back(hit);
            }
          }
        }
      }
    }
  }
  setStatusBit(StatusBit::FIND_HITS_SUCCESSFUL);
  return StatusCode::SUCCESS;
}


StatusCode E3Event::findClusters()
{
  E3HitVecIter hit, hitBegin, hitEnd;
  for (E3::Plane_t plane = 0; plane < E3::kNumPlanes; plane++) {
    hitBegin = m_hitVec[plane].begin();
    hitEnd = m_hitVec[plane].end();
    for (hit = hitBegin; hit != hitEnd; hit++) {
      if (hit == hitBegin || !close(*hit, *(hit - 1))) {
        m_clusterVec[plane].push_back(E3Cluster(plane));
      }
      m_clusterVec[plane].back().addHit(*hit);
    }
  }
  setStatusBit(StatusBit::FIND_CLUSTERS_SUCCESSFUL);
  return StatusCode::SUCCESS;
}


StatusCode E3Event::fitTracks()
{
  E3FittingTool fitTool;
  E3ClusterVecIter topCluster, topBegin, topEnd, midCluster, midBegin, midEnd,
    botCluster, botBegin, botEnd;
  topBegin = m_clusterVec[E3::kTopPlane].begin();
  topEnd = m_clusterVec[E3::kTopPlane].end();
  midBegin = m_clusterVec[E3::kMidPlane].begin();
  midEnd = m_clusterVec[E3::kMidPlane].end();
  botBegin = m_clusterVec[E3::kBotPlane].begin();
  botEnd = m_clusterVec[E3::kBotPlane].end();
  for (topCluster = topBegin; topCluster != topEnd; topCluster ++) {
    for (midCluster = midBegin; midCluster != midEnd; midCluster ++) {
      for (botCluster = botBegin; botCluster != botEnd; botCluster ++) {
        E3PointVec pointVec;
        pointVec.push_back((E3Point)*topCluster);
        pointVec.push_back((E3Point)*midCluster);
        pointVec.push_back((E3Point)*botCluster);
        fitTool.run(pointVec);
        E3Track track = fitTool.track();
        track.setCluster(E3::kTopPlane, *topCluster);
        track.setCluster(E3::kMidPlane, *midCluster);
        track.setCluster(E3::kBotPlane, *botCluster);
        m_trackVec.push_back(track);
      }
    }
  }
  std::sort(m_trackVec.begin(), m_trackVec.end());
  setStatusBit(StatusBit::FIND_TRACKS_SUCCESSFUL);
  return StatusCode::SUCCESS; 
}


std::ostream& E3Event::fillStream(std::ostream& os) const
{
  E3DigiEvent::fillStream(os);
  os << std::endl;
  os << "===================================== "
     << "Hits"
     << " =====================================" << std::endl;
  E3HitVecIter hit, hitBegin, hitEnd;
  for (E3::Plane_t plane = 0; plane < E3::kNumPlanes; plane++) {
    hitBegin = m_hitVec[plane].begin();
    hitEnd = m_hitVec[plane].end();
    for (hit = hitBegin; hit != hitEnd; hit++) {
      os << "--" << *hit << std::endl;
    }
  }
  os << "=================================== "
     << "Clusters"
     << " ===================================" << std::endl;
  E3ClusterVecIter cluster, clusterBegin, clusterEnd;
  for (E3::Plane_t plane = 0; plane < E3::kNumPlanes; plane++) {
    clusterBegin = m_clusterVec[plane].begin();
    clusterEnd = m_clusterVec[plane].end();
    for (cluster = clusterBegin; cluster != clusterEnd; cluster++) {
      os << "--" << *cluster << std::endl;
    }
  }
  os << "==================================== "
     << "Tracks"
     << " ====================================" << std::endl;
  E3TrackVecIter track;
  for (track = m_trackVec.begin(); track != m_trackVec.end(); track++) {
    os << "--" << *track << std::endl;
  }
  os << "================================================================="
     << "===============";
  return os;
}
