/*
  Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include "E3EventList.h"


E3EventList::E3EventList()
{
  clear();
}


void E3EventList::fill(E3BaseEvent event)
{
  push_back(event);
}


void E3EventList::fill(E3EventList list)
{
  LOG(INFO) << "Adding " << list.numEvents() << " event(s) to the event list ("
            << numEvents() << " already in)...";
  E3Chrono chrono;
  std::vector<E3BaseEvent>::insert(end(), list.begin(), list.end());
  LOG(INFO) << "Done in " << chrono.stop() << " s, " << numEvents()
            << " events now in the list.";
}


void E3EventList::insert(const E3BaseEvent &event)
{
  std::vector<E3BaseEvent>::insert(std::lower_bound(begin(), end(), event),
                                   event);
}


void E3EventList::setRunId(E3::Run_t runId){
  for (E3EventList::iterator e = begin(); e != end(); e++) {
    e->setRunId(runId);
  }
}


void E3EventList::setStationId(E3::Station_t stationId){
  for (E3EventList::iterator e = begin(); e != end(); e++) {
    e->setStationId(stationId);
  }
}


void E3EventList::align()
{
  LOG(INFO) << "Aligning event list (" << numEvents() << " events)...";
  E3Chrono chrono;
  for (E3EventList::iterator e = begin(); e != end(); e++) {
    e->setEventId(e - begin());
  }
  LOG(INFO) << "Done in " << chrono.stop() << " s.";
}


void E3EventList::align(E3::Station_t stationId, E3::Run_t runId)
{
  LOG(INFO) << "Aligning event list (" << numEvents() << " events, stationId = "
            << stationId << ", runId = " << runId << ")...";
  E3Chrono chrono;
  for (E3EventList::iterator e = begin(); e != end(); e++) {
    e->setStationId(stationId);
    e->setRunId(runId);
    e->setEventId(e - begin());
  }
  LOG(INFO) << "Done in " << chrono.stop() << " s.";
}


void E3EventList::fillRandom(double rate, double tstart, double tstop, int seed,
                             E3::Station_t stationId, E3::Run_t runId)
{
  LOG(INFO) << "Filling event list (rate = " << rate << " Hz, start = " 
            << tstart << " s, stop = " << tstop << " s, seed = " << seed
            << ")...";
  E3Chrono chrono;
  E3RandomTimeGenerator randgen(rate, tstart, seed);
  E3Timestamp stop(tstop);
  E3Timestamp t = randgen.timestamp();
  long eventId = numEvents();
  while (t < stop) {
    fill(E3BaseEvent(stationId, runId, eventId, t));
    eventId ++;
    t = randgen.timestamp();
  }
  LOG(INFO) << "Done in " << chrono.stop() << " s, " << numEvents()
            << " event(s) added.";
}


E3EventList E3EventList::smear(double sigma, int seed) const
{
  LOG(INFO) << "Smearing event list (" << numEvents() << " events, sigma = "
            << sigma << ")...";
  E3Chrono chrono;
  E3EventList eventList;
  E3RandomGaussGenerator randgen(0., sigma, seed);
  for (E3EventList::const_iterator e = begin(); e != end(); e++) {
    E3Timestamp t = (*e).timestamp();
    t += randgen.sample();
    E3BaseEvent event((*e).stationId(), (*e).runId(), (*e).eventId(), t);
    eventList.fill(event);
  }
  LOG(INFO) << "Done in " << chrono.stop() << " s.";
  return eventList;
}


void E3EventList::sort()
{
  LOG(INFO) << "Sorting event list (" << numEvents() << " events)...";
  E3Chrono chrono;
  std::sort(begin(), end());
  LOG(INFO) << "Done in " << chrono.stop() << " s.";
}


unsigned E3EventList::bisect(const E3BaseEvent &event) const
{
  E3EventList::const_iterator lower = std::lower_bound(begin(), end(), event);
  return (lower - begin());
}


std::ostream& E3EventList::fillStream(std::ostream& os, unsigned n) const
{
  for (E3EventList::const_iterator event = begin(); event != end(); event ++) {
    unsigned offset = event - begin();
    os << *event;
    if (event < end() - 1) os << std::endl;
    if (offset >= n - 1 && offset < (numEvents() - n)) {
      event += numEvents() - 2*n;
      if (numEvents() != 2*n) os << "..." << std::endl;
    }
  }
  return os;
}



E3CoincidenceList E3EventList::searchCoincidences(double window)
{
  LOG(INFO) << "Searching for coincidences...";
  E3Chrono chrono;
  E3CoincidenceList coincidenceList;
  E3EventList::const_iterator event;
  (*this).sort();
  for (event = (*this).begin(); event != (*this).end() - 1; event++) {
    if (event->stationId() != (event + 1)->stationId() &&
        distance(*event, *(event + 1)) < window) {
      coincidenceList.fill(*event, *(event + 1));
    }
  }
  LOG(INFO) << "Done in " << chrono.stop() << " s, " << coincidenceList.size()
            << " coincidences found.";
  return coincidenceList;
}


E3CoincidenceList match(const E3EventList &list1, const E3EventList &list2,
                        double window)
{
  LOG(INFO) << "Merging event lists for matching...";
  E3EventList list;
  list.reserve(list1.numEvents() + list2.numEvents());
  list.fill(list1);
  list.fill(list2);
  return list.searchCoincidences(window);
}
