/*
  Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef E3Option_H
#define E3Option_H

#include <string>
#include <iostream>
#include <iomanip>

#include "E3Variant.h"


/*! \class E3Option
  \brief Class describing a command-line option.

  A command-line option has a long form, a short form, a value and a help
  string. Additionally, we pass along two flags specifying whether a given
  option is initialized (i.e. has a meaningful value) and whether it is
  required (i.e. it has to be specified from the command line.)

  The tricky part in the option class is that the value can be be of several
  different types---bool, int floating point numbers or strings. That's the
  whole purpose of the E3Variant class used here.

  Ideally E3Option should really be a template class (as E3Variant should),
  but that adds complication when it comes to store options in some standard
  container in the E3OptionParser class. That's why we went the non-template
  way.
*/


class E3Option
{
 public:
  /// \brief Constructor.
  E3Option(std::string longopt, char shortopt, E3Variant value,
           std::string help, bool required = false, bool initialized = true);

  /// \brief Destructor.
  ~E3Option() {}

  /// \brief Return the long form of the option.
  std::string longOpt() const { return m_longOpt; }

  /// \brief Return the short form of the option.
  char shortOpt() const { return m_shortOpt; }

  /// \brief Return true if the option is required.
  bool required() const { return m_required; }

  /// \brief Return true if the option is initialized.
  bool initialized() const { return m_initialized; }

  /// \brief Return true if the option is not initialized.
  bool uninitialized() const { return !m_initialized; }
  
  /// \brief Return the option value.
  template <class T> T value() const { return m_value.value<T>(); }

  /// \brief Set the option value.
  void setValue(E3Variant value)
  { m_value = value; m_initialized = true; }

  /// \brief Return the option type.
  E3Variant::Type type() const { return m_value.type(); }

  /// \brief Return the string version of the option type.
  std::string strtype() const { return m_value.strtype(); }

  /// \brief Standard output.
  std::ostream& fillStream(std::ostream& os) const;

  /// \brief Standard output.
  friend std::ostream& operator<<(std::ostream& os, const E3Option& o)
  { return o.fillStream(os); }

 private:
  /// \brief The long version of the option.
  std::string m_longOpt;

  /// \brief The short version of the option.
  char m_shortOpt;

  /// \brief The option value.
  E3Variant m_value;

  /// \brief The help string.
  std::string m_help;

  /// \brief Flag telling whether the option is required.
  bool m_required;

  /// \brief Flag telling whether the option value is initialized.
  bool m_initialized;
};

#endif
