/*
  Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include "E3OptionParser.h"


E3OptionParser::E3OptionParser(std::string intent, std::string usage) :
  m_intent(intent),
  m_usage(usage)
{
  addOption<bool>("help", 'h', "Print help and exit");
}


void E3OptionParser::addOption(std::string longopt, char shortopt,
                               E3Option option)
{
  m_optionMap.insert(std::make_pair(longopt, option));
  m_short2longMap.insert(std::make_pair(shortopt, longopt));
}


std::string E3OptionParser::getLongOption(char shortopt) const
{
  std::map<char, std::string>::const_iterator item =
    m_short2longMap.find(shortopt);
  if (item == m_short2longMap.end()) {
    std::cout << shortopt << std::endl;
    parseError("Unrecognized short option -" + std::string(1, shortopt) + ".");
  }
  return item->second;
}


void E3OptionParser::parseError(std::string msg) const
{
  std::cout << *this << std::endl << "Parsing error: " << msg << std::endl;
  exit(1);
}


void E3OptionParser::help() const
{
  std::cout << *this << std::endl;
  exit(0);
}


void E3OptionParser::checkOptions() const 
{
  std::map<std::string, E3Option>::const_iterator item;
  for (item = m_optionMap.begin(); item != m_optionMap.end(); item++) {
    E3Option option = item->second;
    if (option.required() && option.uninitialized()) {
      std::string msg = "Option '" + option.longOpt() + "' is required.";
      parseError(msg);
    }
  }
}


std::map<std::string, E3Option>::iterator
E3OptionParser::getMapIter(std::string longopt)
{
  std::map<std::string, E3Option>::iterator item = m_optionMap.find(longopt);
  if (item == m_optionMap.end()) {
    parseError("Unrecognized long option --" + longopt + ".");
  }
  return item;
}


void E3OptionParser::parse(int argc, char* argv[])
{
  std::string token, lopt;
  for (int i = 1; i < argc; i++) {
    token = argv[i];
    // Need help?
    if (token == "-h" || token == "--help") help();
    // Resolve the mapping between long and short options.
    if (token.substr(0,2) == "--") {
      lopt = token.substr(2, token.length());
    } else if (token.substr(0,1) == "-") {
      lopt = getLongOption(argv[i][1]);
    }
    if (token.substr(0,1) == "-") {
      // Is the token an option...
      std::map<std::string, E3Option>::iterator item = getMapIter(lopt);
      if (item->second.type() == E3Variant::BOOL) {
        item->second.setValue(E3Variant(true));
      } else {
        i++;
        if (i >= argc) {
          parseError("Missing argument for option " + lopt + ".");
        }
        token = argv[i];
        if (item->second.type() == E3Variant::STRING) {
          item->second.setValue(E3Variant(token));
        }
        if (item->second.type() == E3Variant::INT) {
          try { item->second.setValue(E3Variant(atoi(token.c_str()))); }
          catch (std::exception& e) {
            parseError("Invalid argument '" + token + "' to '" + lopt + "'" );
          }
        }
        if (item->second.type() == E3Variant::DOUBLE) {
          try { item->second.setValue(E3Variant(atof(token.c_str()))); }
          catch (std::exception& e) {
            parseError("Invalid argument '" + token + "' to '" + lopt + "'" );
          }
        }
      }      
    } else {
      // ... or an argument?
      m_argumentVec.push_back(token); 
    }
  }
  checkOptions();
}


std::ostream& E3OptionParser::fillStream(std::ostream& os) const
{
  os << m_intent << "\n\n"
     << "Usage: " << m_usage << "\n\n"
     << "Recognized options: " << std::endl;
  std::map<std::string, E3Option>::const_iterator item;
  for (item = m_optionMap.begin(); item != m_optionMap.end(); item++) {
    os << "  " << item->second << std::endl;
  }
  return os;
}
