/*
  Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef E3OptionParser_H
#define E3OptionParser_H

#include <iostream>
#include <string>
#include <vector>
#include <map>
#include <iomanip>
#include <exception>
#include <stdlib.h>

#include "E3Option.h"


/*! \class E3OptionParser
  \brief Class for parsing command-line options.
*/

class E3OptionParser
{
 public:
  /// \brief Constructor
  E3OptionParser(std::string intent, std::string usage);

  /// \brief Destructor.
  ~E3OptionParser() {}

  /// \brief Add an option to the option map.
  void addOption(std::string longopt, char shortopt, E3Option option);

  /// \brief Add an option to the option map.
  template <class T> void addOption(std::string longopt, char shortopt,
                                    T defaultValue, std::string help)
    {
      E3Option option(longopt, shortopt, E3Variant(defaultValue), help);
      addOption(longopt, shortopt, option);
    }

  /// \brief Add an option with no default value.
  template <class T> void addOption(std::string longopt, char shortopt,
                                    std::string help, bool required);

  /// \brief Add a non-required option with no degault value
  template <class T> void addOption(std::string longopt, char shortopt,
                                    std::string help);

  /// \brief Return the *value* of a given option.
  template <class T> T getOption(std::string longopt)
    { 
      std::map<std::string, E3Option>::iterator item = getMapIter(longopt);
      return (item->second).value<T>();
    }

  /// \brief Return the argument list.
  std::vector<std::string> getArguments() const { return m_argumentVec; }

  /// \brif Return the number of arguments.
  unsigned int numArguments() const { return m_argumentVec.size(); }

  /// \brief Print out a parser error.
  void parseError(std::string msg) const;

  /// \brief Print out help and exit.
  void help() const;

  /// \brief Parse the arguments.
  void parse(int argc, char* argv[]);

  /// \brief Standard output.
  std::ostream& fillStream(std::ostream& os) const;

  /// \brief Standard output.
  friend std::ostream& operator<<(std::ostream& os, const E3OptionParser& p)
  { return p.fillStream(os); }

 private:
  /// \brief Check whether all the required options have been set.
  void checkOptions() const ;

  /// \brief Return the long option corresponding to a short option.
  std::string getLongOption(char shortopt) const;

  /// \brief Access map elements by long name.
  std::map<std::string, E3Option>::iterator getMapIter(std::string longopt);

  /// \brief The program intent.
  std::string m_intent;

  /// \brief The program usage.
  std::string m_usage;

  /// \brief The map storing the corrispondence between the long and the
  /// short forms of the options.
  std::map<char, std::string> m_short2longMap;

  /// \brief The underlying option map.
  std::map<std::string, E3Option> m_optionMap;

  /// \brief The underlying argument list.
  std::vector<std::string> m_argumentVec;
};



template <> inline void E3OptionParser::addOption<bool>(std::string longopt,
                                                        char shortopt,
                                                        std::string help)
{
  E3OptionParser::addOption<bool>(longopt, shortopt, false, help);
}


template <> inline void E3OptionParser::addOption<int>(std::string longopt,
                                                       char shortopt,
                                                       std::string help,
                                                       bool required)
{
  E3Option option(longopt, shortopt, E3Variant(0), help, required, false);
  E3OptionParser::addOption(longopt, shortopt, option);
}


template <> inline void E3OptionParser::addOption<double>(std::string longopt,
                                                          char shortopt,
                                                          std::string help,
                                                          bool required)
{
  E3Option option(longopt, shortopt, E3Variant(0.), help, required, false);
  E3OptionParser::addOption(longopt, shortopt, option);
}


template <> inline void E3OptionParser::addOption<std::string>(std::string longopt,
                                                               char shortopt,
                                                               std::string help,
                                                               bool required)
{
  E3Option option(longopt, shortopt, E3Variant(std::string("")), help,
                  required, false);
  E3OptionParser::addOption(longopt, shortopt, option);
}


#endif
