/*
  Copyright (C) 2012--2013 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include "E3TelescopeConfiguration.h"


E3TelescopeConfiguration::E3TelescopeConfiguration()
{
  setDefaults();
}


void E3TelescopeConfiguration::setDefaults()
{
  LOG(INFO) << "Setting up default telescope configuration...";
  m_z[E3::kTopPlane] = 120.;
  m_z[E3::kMidPlane] = 60.;
  m_z[E3::kBotPlane] = 0.;
  m_layerThickness = 0.;
  m_orientation = 0.;
  m_cableDelay[E3::kTopPlane][E3::kPosEnd] = 0;
  m_cableDelay[E3::kTopPlane][E3::kNegEnd] = 0;
  m_cableDelay[E3::kMidPlane][E3::kPosEnd] = 0;
  m_cableDelay[E3::kMidPlane][E3::kNegEnd] = -4250;
  m_cableDelay[E3::kBotPlane][E3::kPosEnd] = 0;
  m_cableDelay[E3::kBotPlane][E3::kNegEnd] = 0;
  m_numStripsPerPlane = 24;
  m_stripPitch = 3.875;
  m_stripLength = 220.;
  m_propSpeed = 0.015;
}


StatusCode E3TelescopeConfiguration::read(const char* filePath)
{
  LOG(INFO) << "Reading configuration file...";
  E3TextInputFile inputFile;
  StatusCode sc = inputFile.open(filePath);
  if (sc) return sc;
  std::string token;
  while (inputFile >> token) {
    if (token == "z0")
      inputFile >> token >> m_z[E3::kTopPlane];
    else if (token == "z1")
      inputFile >> token >> m_z[E3::kMidPlane];
    else if (token == "z2")
      inputFile >> token >> m_z[E3::kBotPlane];
    else if (token == "Orientation")
      inputFile >> token >> m_orientation;
    else if (token == "dz")
      inputFile >> token >> m_layerThickness;
    else if (token == "Strip_pitch")
      inputFile >> token >> m_stripPitch;
    else if (token == "Strip_length")
      inputFile >> token >> m_stripLength;
    else if (token == "Delay_top_pos") 
      inputFile >> token >> m_cableDelay[E3::kTopPlane][E3::kPosEnd];
    else if (token == "Delay_top_neg") 
      inputFile >> token >> m_cableDelay[E3::kTopPlane][E3::kNegEnd];
    else if (token == "Delay_mid_pos") 
      inputFile >> token >> m_cableDelay[E3::kMidPlane][E3::kPosEnd];
    else if (token == "Delay_mid_neg") 
      inputFile >> token >> m_cableDelay[E3::kMidPlane][E3::kNegEnd];
    else if (token == "Delay_bot_pos") 
      inputFile >> token >> m_cableDelay[E3::kBotPlane][E3::kPosEnd];
    else if (token == "Delay_bot_neg") 
      inputFile >> token >> m_cableDelay[E3::kBotPlane][E3::kNegEnd];
  }
  inputFile.close();
  LOG(INFO) << "Configuration file successfully processed.";
  return sc;
}


StatusCode E3TelescopeConfiguration::read(const std::string filePath)
{
  return read(filePath.c_str());
}


std::ostream& E3TelescopeConfiguration::fillStream(std::ostream& os) const
{
  os << E3TerminalFormatting::title("Telescope configuration", true)
     << E3TerminalFormatting::keyval("Top layer z", m_z[E3::kTopPlane], "cm")
     << E3TerminalFormatting::keyval("Mid layer z", m_z[E3::kMidPlane], "cm")
     << E3TerminalFormatting::keyval("Bottom layer z", m_z[E3::kBotPlane], "cm")
     << E3TerminalFormatting::keyval("Orientation", m_orientation, "deg")
     << E3TerminalFormatting::keyval("Layer thickness", m_layerThickness, "cm")
     << E3TerminalFormatting::keyval("Strips per plane", m_numStripsPerPlane)
     << E3TerminalFormatting::keyval("Strip pitch", m_stripPitch, "cm")
     << E3TerminalFormatting::keyval("Strip length", m_stripLength, "cm")
     << E3TerminalFormatting::keyval("Propagation speed", m_propSpeed, "cm/ps");
  for (E3::Plane_t plane = 0; plane < E3::kNumPlanes; plane++) {
    std::stringstream key("");
    key << "Cable delay for plane " << plane;
    std::stringstream value("");
    value << m_cableDelay[plane][E3::kPosEnd] << " ps [+ side], "
	  << m_cableDelay[plane][E3::kNegEnd] << " ps [- side]";
    os << E3TerminalFormatting::keyval(key.str(), value.str());
  }
  os << E3TerminalFormatting::hline();
  return os;
}
