/*
  Copyright (C) 2013 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef E3TextNtuple_H
#define E3TextNtuple_H


#include <string>
#include <vector>

#include "E3TextOutputFile.h"
#include "E3Event.h"


/*! \class E3TextNtuple
  \brief Utility class describing a text plain ntuple.
*/

class E3TextNtuple : public E3TextOutputFile
{
 public:
  /// \brief Default constructor.
  E3TextNtuple(std::string label);

  /// \brief Alternative constructor given a file path.
  StatusCode open(const char* filePath);

  /// \brief Alternative constructor given a file path.
  StatusCode open(const std::string filePath);

  /// \brief Destructor.
  ~E3TextNtuple() {}

  /// \brief Add a column to the ntuple.
  ///
  /// Here we add a std::pair to the m_columns class member. Note that this is
  /// only used to ensure the ntuple header is written out in a consistent
  /// fashion, so that it can be parsed later and converted to other formats
  /// (e.g., ROOT).
  void addColumn(std::string name, char type);

  /// \brief Pure virtual method to be reimplemented in derived classes.
  ///
  /// This is called in the constructors accepting the output file path as
  /// a parameter.
  virtual void addColumns() = 0;

  /// \brief Write the header information to the output text file.
  virtual void writeHeader();

  /// \brief Write the eol character to file.
  inline void endline() { *this << m_eol; }

  /// \brief Write a single number to the output file.
  template <class T> void write(T value, unsigned short width);

  /// \brief Write a single number to the output file.
  template <class T> void write(T value, unsigned short width,
                                unsigned short precision);

 protected:
  /// A text label identifying the ntuple.
  std::string m_label;

  /// \brief The basic column descriptors.
  std::vector<std::pair<std::string, char>> m_columns;

  /// \brief The column separator.
  ///
  /// This is really only used for writing out the ntuple header, as the
  /// field separation in the actual ntuple rows is achieved via ostream
  /// manipulators.
  std::string m_separator;

  /// \brief The row end-of-line string.
  ///
  /// Keep in mind the difference between Linux and Windows (i.e. plain LF
  /// vs. CR+LF).
  std::string m_eol;
};


template <class T> void E3TextNtuple::write(T value, unsigned short width)
{
  *this << std::setw(width) << value;
}


template <class T> void E3TextNtuple::write(T value, unsigned short width,
                                            unsigned short precision)
{
  *this << std::setw(width) << std::setprecision(precision) << value;
}

#endif
