/*
  Copyright (C) 2012 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#ifndef E3Variant_H
#define E3Variant_H

#include <iostream>
#include <typeinfo>
#include <string>


/*! \class E3Variant
  \brief Poor man's implementation of a c++ variant type.
*/


class E3Variant
{
 public:
  enum Type {
    BOOL,
    INT,
    DOUBLE,
    STRING
  };
  
  /// \brief Constructor setting the variant value.
  template <class T> E3Variant(T value);

  /// \brief Destructor.
  ~E3Variant() {}

  /// \brief Return the type currently held in the variable.
  Type type() const { return m_type; }

  /// \brief Return a string representation of the variant type.
  std::string strtype() const;

  /// \brief Forward declaration for the specialized template functions.
  template<class T> T value() const;

  /// \brief Standard output.
  std::ostream& fillStream(std::ostream& os) const;

  /// \brief Standard output.
  friend std::ostream& operator<<(std::ostream& os, const E3Variant& v)
  { return v.fillStream(os); }

 private:
  /// \brief The basic underlying class member storing the variable value.
  union Data {
    bool b;
    int i;
    double d;
  } m_data;
  
  /// \brief The member variable for the string data of the variant.
  std::string m_stringData;
  
  /// brief The type currently held in the variant.
  Type m_type;
};


template <> inline E3Variant::E3Variant<bool>(bool value)
{ m_data.b = value; m_type = BOOL; }

template <> inline E3Variant::E3Variant<int>(int value)
{ m_data.i = value; m_type = INT; }

template <> inline E3Variant::E3Variant<double>(double value)
{ m_data.d = value; m_type = DOUBLE; }

template <> inline E3Variant::E3Variant<std::string>(std::string value)
{ m_stringData = value; m_type = STRING; }

template <> bool inline E3Variant::value<bool>() const
{ return m_data.b; }  

template <> int inline E3Variant::value<int>() const
{ return m_data.i; }

template <> double inline E3Variant::value<double>()
const { return m_data.d; }

template <> std::string inline E3Variant::value<std::string>() const
{ return m_stringData; }

#endif
