/*
  Copyright (C) 2013 Luca Baldini (luca.baldini@pi.infn.it)

  For the license terms see the file LICENSE, distributed along with this
  software.

  This program is free software; you can redistribute it and/or modify it under
  the terms of the GNU General Public License as published by the Free Software
  Foundation; either version 3 of the License, or (at your option) any later
  version.
  
  This program is distributed in the hope that it will be useful, but WITHOUT
  ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
  FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License along with
  this program; if not, write to the Free Software Foundation, Inc., 51
  Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
*/

#include "E3RawDataFile.h"
#include "E3TextOutputFile.h"
#include "E3Option.h"
#include "E3OptionParser.h"
#include "E3Event.h"
#include "E3MessageLogger.h"
#include "E3TelescopeConfiguration.h"
#include "E3Chrono.h"
#include "E3CernNtuple.h"
#include "E3SummaryNtuple.h"
#include "E3TerminalFormatting.h"


int main (int argc, char* argv[]) {

  E3TerminalFormatting::startmsg();

  // Setup command-line options.
  std::string intent =
    "Read a binary data (.dat) file and run the full reconstruction.";
  std::string usage = "e3runrecon [options] file.dat";
  E3OptionParser parser(intent, usage);
  parser.addOption<int>("num-events", 'n', -1,
                        "Number of events to be processed");
  parser.addOption<std::string>("config-file", 'c',
                                "Path to the input configuration file", false);
  parser.addOption<std::string>("cern-ntuple", 'C',
                                "Path to the output CERN-style ntuple", false);
  parser.addOption<std::string>("summary-ntuple", 'S',
                                "Path to the output summary ntuple", false);
  parser.parse(argc, argv);
  if (parser.numArguments() != 1) {
    parser.parseError("Please provide exactly one input .dat file.");
  }

  // Parse options.
  std::string inputFilePath = parser.getArguments().front();
  const int numEvents = parser.getOption<int>("num-events");
  std::string cernFilePath = parser.getOption<std::string>("cern-ntuple");
  if (cernFilePath.size() == 0) {
    cernFilePath = inputFilePath;
    cernFilePath.replace(cernFilePath.end() - 4, cernFilePath.end(),
                         "_cerntuple.out");
  }
  std::string summaryFilePath = parser.getOption<std::string>("summary-ntuple");
  if (summaryFilePath.size() == 0) {
    summaryFilePath = inputFilePath;
    summaryFilePath.replace(summaryFilePath.end() - 4, cernFilePath.end(),
                            "_stdtuple.out");
  }

  // @todo Mind this must be read from the proper configuration file.
  std::string configFilePath = parser.getOption<std::string>("config-file");
  E3TelescopeConfiguration configuration;
  if (configFilePath.size()) {
    configuration.read(configFilePath);
  }
  LOG(INFO) << "Full dump of configuration following...\n" << configuration;

  // Try and open the input file.
  E3RawDataFile inputFile;
  if (inputFile.open(inputFilePath)) return(1);

  // Read the file header.
  E3RunInfo runInfo = inputFile.parseHeader();

  // Initialize the CERN ntuple.
  E3CernNtuple cernNtuple;
  if (cernNtuple.open(cernFilePath)) return(1);

  // Initialize the summary ntuple (if needed).
  E3SummaryNtuple summaryNtuple;
  if (summaryNtuple.open(summaryFilePath)) return(1);

  // Loop through the events.
  E3Chrono chrono;
  LOG(INFO) << "Starting event loop...";
  int i = 1;
  while (i != numEvents) {
    E3Event event = inputFile.next();
    if (event.unpack(runInfo) == StatusCode::SUCCESS) {
      event.reconstruct(configuration);
    }
    if (!event.hasErrors() && event.numTracks()) cernNtuple.writeRow(event);
    summaryNtuple.writeRow(event);
    if (inputFile.numBytesLeft() == 0) break;
    i++;
  }
  LOG(INFO) << "Event loop finished, " << i << " event(s) processed in "
            << chrono.stop() << " s.";
  return 0;
}
